diff mbox

DA850 - SD detection broken by edma change (but only on mainline)

Message ID 1397458847.3064.15.camel@localhost.localdomain (mailing list archive)
State New, archived
Headers show

Commit Message

Peter Howard April 14, 2014, 7 a.m. UTC
For the DA850, I've found that trying to detection of a card in the SD
slot during boot is broken as of 3.12 on mainline.  You end up like this
(from a 3.14 kernel.org kernel):

...
davinci_mdio davinci_mdio.0: davinci mdio revision 1.5                          
davinci_mdio davinci_mdio.0: detected phy mask fffffffe                         
libphy: davinci_mdio.0: probed                                                  
davinci_mdio davinci_mdio.0: phy[0]: device davinci_mdio-0:00, driver unknown   
input: gpio-keys-polled as /devices/platform/gpio-keys-polled.1/input/input0    
i2c /dev entries driver                                                         
davinci_mmc da830-mmc.0: Using DMA, 4-bit mode                                  
TCP: cubic registered                                                           
NET: Registered protocol family 17                                              
console [netcon0] enabled                                                       
netconsole: network logging started                                             
davinci_emac davinci_emac.1: using random MAC addr: ba:64:c2:e7:17:f9           
Waiting for root device /dev/mmcblk0p2...                                       
random: nonblocking pool is initialized                                         

and then nothing . . . 

as opposed to previously:

libphy: davinci_mdio.0: probed                                                  
davinci_mdio davinci_mdio.0: phy[0]: device davinci_mdio-0:00, driver unknown   
input: gpio-keys-polled as /devices/platform/gpio-keys-polled.1/input/input0    
i2c /dev entries driver                                                         
edma-dma-engine edma-dma-engine.0: allocated channel for 0:17                   
edma-dma-engine edma-dma-engine.0: allocated channel for 0:16                   
davinci_mmc da830-mmc.0: Using DMA, 4-bit mode                                  
TCP: cubic registered                                                           
NET: Registered protocol family 17                                              
console [netcon0] enabled                                                       
netconsole: network logging started                                             
davinci_emac davinci_emac.1: using random MAC addr: 2a:e7:9f:05:6c:c9           
Waiting for root device /dev/mmcblk0p2...                                       
mmc0: new high speed SDHC card at address e624                                  
mmcblk0: mmc0:e624 SU08G 7.40 GiB                                               
 mmcblk0: p1 p2                                                                 
kjournald starting.  Commit interval 5 seconds                                  

...

I traced the problem to the following commit.  Interestingly, it is
_not_ in the linux-davinci repo yet.

Joel Fernandes - given the commit is yours, any ideas why it breaks the
DA850?

(Note: although in that failure you don't see channel allocation
messages for the edma engine, I have seen failures building immediately
around the commit below where those allocation messages do appear (but
the card isn't detected) 

commit 534070622d2c7fbc5cc929aa93541ccd0ae52ab1
Author: Joel Fernandes <joelf@ti.com>
Date:   Tue Sep 3 10:02:46 2013 -0500

    dma: edma: Write out and handle MAX_NR_SG at a given time
    
    Process SG-elements in batches of MAX_NR_SG if they are greater
    than MAX_NR_SG. Due to this, at any given time only those many
    slots will be used in the given channel no matter how long the
    scatter list is. We keep track of how much has been written
    inorder to process the next batch of elements in the scatter-list
    and detect completion.
    
    For such intermediate transfer completions (one batch of MAX_NR_SG),
    make use of pause and resume functions instead of start and stop
    when such intermediate transfer is in progress or completed as we
    donot want to clear any pending events.
    
    Signed-off-by: Joel Fernandes <joelf@ti.com>
    Signed-off-by: Vinod Koul <vinod.koul@intel.com>

Comments

Sekhar Nori April 14, 2014, 8:32 a.m. UTC | #1
Peter,

On Monday 14 April 2014 12:30 PM, Peter Howard wrote:
> For the DA850, I've found that trying to detection of a card in the SD
> slot during boot is broken as of 3.12 on mainline.  You end up like this
> (from a 3.14 kernel.org kernel):

Could you try this patch?

http://www.spinics.net/lists/linux-omap/msg104627.html

Thanks,
Sekhar
Peter Howard April 14, 2014, 10:34 p.m. UTC | #2
On Mon, 2014-04-14 at 14:02 +0530, Sekhar Nori wrote:
> Peter,
> 
> On Monday 14 April 2014 12:30 PM, Peter Howard wrote:
> > For the DA850, I've found that trying to detection of a card in the SD
> > slot during boot is broken as of 3.12 on mainline.  You end up like this
> > (from a 3.14 kernel.org kernel):
> 
> Could you try this patch?
> 
> http://www.spinics.net/lists/linux-omap/msg104627.html
> 

That patch, by itself, doesn't appear to fix the problem.  Whereas fully
reverting the quoted patch does.
Peter Howard April 16, 2014, 11:20 p.m. UTC | #3
On Tue, 2014-04-15 at 08:34 +1000, Peter Howard wrote:
> On Mon, 2014-04-14 at 14:02 +0530, Sekhar Nori wrote:
> > Peter,
> > 
> > On Monday 14 April 2014 12:30 PM, Peter Howard wrote:
> > > For the DA850, I've found that trying to detection of a card in the SD
> > > slot during boot is broken as of 3.12 on mainline.  You end up like this
> > > (from a 3.14 kernel.org kernel):
> > 
> > Could you try this patch?
> > 
> > http://www.spinics.net/lists/linux-omap/msg104627.html
> > 
> 
> That patch, by itself, doesn't appear to fix the problem.  Whereas fully
> reverting the quoted patch does.

Correction.  I don't know what I did wrong the first time, but I tried
applying it again to a clean untar of the linux-3.14 tarball from
kernel.org, and it _does_ fix the problem.

Thanks.
Sekhar Nori April 21, 2014, 6:17 a.m. UTC | #4
On Thursday 17 April 2014 04:50 AM, Peter Howard wrote:
> On Tue, 2014-04-15 at 08:34 +1000, Peter Howard wrote:
>> On Mon, 2014-04-14 at 14:02 +0530, Sekhar Nori wrote:
>>> Peter,
>>>
>>> On Monday 14 April 2014 12:30 PM, Peter Howard wrote:
>>>> For the DA850, I've found that trying to detection of a card in the SD
>>>> slot during boot is broken as of 3.12 on mainline.  You end up like this
>>>> (from a 3.14 kernel.org kernel):
>>>
>>> Could you try this patch?
>>>
>>> http://www.spinics.net/lists/linux-omap/msg104627.html
>>>
>>
>> That patch, by itself, doesn't appear to fix the problem.  Whereas fully
>> reverting the quoted patch does.
> 
> Correction.  I don't know what I did wrong the first time, but I tried
> applying it again to a clean untar of the linux-3.14 tarball from
> kernel.org, and it _does_ fix the problem.

Okay, cool. The patch is present in v3.15-rc2 with stable tag added so
it should fan out to various stable trees soon.

Thanks,
Sekhar
diff mbox

Patch

diff --git a/drivers/dma/edma.c b/drivers/dma/edma.c
index f907512..4c1c258 100644
--- a/drivers/dma/edma.c
+++ b/drivers/dma/edma.c
@@ -56,6 +56,7 @@  struct edma_desc {
 	struct list_head		node;
 	int				absync;
 	int				pset_nr;
+	int				processed;
 	struct edmacc_param		pset[0];
 };
 
@@ -104,22 +105,34 @@  static void edma_desc_free(struct virt_dma_desc *vdesc)
 /* Dispatch a queued descriptor to the controller (caller holds lock) */
 static void edma_execute(struct edma_chan *echan)
 {
-	struct virt_dma_desc *vdesc = vchan_next_desc(&echan->vchan);
+	struct virt_dma_desc *vdesc;
 	struct edma_desc *edesc;
-	int i;
-
-	if (!vdesc) {
-		echan->edesc = NULL;
-		return;
+	struct device *dev = echan->vchan.chan.device->dev;
+	int i, j, left, nslots;
+
+	/* If either we processed all psets or we're still not started */
+	if (!echan->edesc ||
+	    echan->edesc->pset_nr == echan->edesc->processed) {
+		/* Get next vdesc */
+		vdesc = vchan_next_desc(&echan->vchan);
+		if (!vdesc) {
+			echan->edesc = NULL;
+			return;
+		}
+		list_del(&vdesc->node);
+		echan->edesc = to_edma_desc(&vdesc->tx);
 	}
 
-	list_del(&vdesc->node);
+	edesc = echan->edesc;
 
-	echan->edesc = edesc = to_edma_desc(&vdesc->tx);
+	/* Find out how many left */
+	left = edesc->pset_nr - edesc->processed;
+	nslots = min(MAX_NR_SG, left);
 
 	/* Write descriptor PaRAM set(s) */
-	for (i = 0; i < edesc->pset_nr; i++) {
-		edma_write_slot(echan->slot[i], &edesc->pset[i]);
+	for (i = 0; i < nslots; i++) {
+		j = i + edesc->processed;
+		edma_write_slot(echan->slot[i], &edesc->pset[j]);
 		dev_dbg(echan->vchan.chan.device->dev,
 			"\n pset[%d]:\n"
 			"  chnum\t%d\n"
@@ -132,24 +145,31 @@  static void edma_execute(struct edma_chan *echan)
 			"  bidx\t%08x\n"
 			"  cidx\t%08x\n"
 			"  lkrld\t%08x\n",
-			i, echan->ch_num, echan->slot[i],
-			edesc->pset[i].opt,
-			edesc->pset[i].src,
-			edesc->pset[i].dst,
-			edesc->pset[i].a_b_cnt,
-			edesc->pset[i].ccnt,
-			edesc->pset[i].src_dst_bidx,
-			edesc->pset[i].src_dst_cidx,
-			edesc->pset[i].link_bcntrld);
+			j, echan->ch_num, echan->slot[i],
+			edesc->pset[j].opt,
+			edesc->pset[j].src,
+			edesc->pset[j].dst,
+			edesc->pset[j].a_b_cnt,
+			edesc->pset[j].ccnt,
+			edesc->pset[j].src_dst_bidx,
+			edesc->pset[j].src_dst_cidx,
+			edesc->pset[j].link_bcntrld);
 		/* Link to the previous slot if not the last set */
-		if (i != (edesc->pset_nr - 1))
+		if (i != (nslots - 1))
 			edma_link(echan->slot[i], echan->slot[i+1]);
 		/* Final pset links to the dummy pset */
 		else
 			edma_link(echan->slot[i], echan->ecc->dummy_slot);
 	}
 
-	edma_start(echan->ch_num);
+	edesc->processed += nslots;
+
+	edma_resume(echan->ch_num);
+
+	if (edesc->processed <= MAX_NR_SG) {
+		dev_dbg(dev, "first transfer starting %d\n", echan->ch_num);
+		edma_start(echan->ch_num);
+	}
 }
 
 static int edma_terminate_all(struct edma_chan *echan)
@@ -368,19 +388,24 @@  static void edma_callback(unsigned ch_num, u16 ch_status, void *data)
 	struct edma_desc *edesc;
 	unsigned long flags;
 
-	/* Stop the channel */
-	edma_stop(echan->ch_num);
+	/* Pause the channel */
+	edma_pause(echan->ch_num);
 
 	switch (ch_status) {
 	case DMA_COMPLETE:
-		dev_dbg(dev, "transfer complete on channel %d\n", ch_num);
-
 		spin_lock_irqsave(&echan->vchan.lock, flags);
 
 		edesc = echan->edesc;
 		if (edesc) {
+			if (edesc->processed == edesc->pset_nr) {
+				dev_dbg(dev, "Transfer complete, stopping channel %d\n", ch_num);
+				edma_stop(echan->ch_num);
+				vchan_cookie_complete(&edesc->vdesc);
+			} else {
+				dev_dbg(dev, "Intermediate transfer complete on channel %d\n", ch_num);
+			}
+
 			edma_execute(echan);
-			vchan_cookie_complete(&edesc->vdesc);
 		}
 
 		spin_unlock_irqrestore(&echan->vchan.lock, flags);